@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (c) 2012-2021 Snowflake Computing Inc. All right reserved.
+# Copyright (c) 2012-2021 Snowflake Computing Inc. All rights reserved.
 #
 
 import binascii
@@ -441,7 +441,7 @@ class SnowflakeConverter(object):
             + "000"
         )
 
-    def to_snowflake(self, value):
+    def to_snowflake(self, value: Any) -> Any:
         """Converts Python data to Snowflake data for pyformat/format style.
 
         The output is bound in a query in the client side.
