@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (c) 2012-2021 Snowflake Computing Inc. All right reserved.
+# Copyright (c) 2012-2021 Snowflake Computing Inc. All rights reserved.
 #
 
 import logging
@@ -10,19 +10,39 @@ import signal
 import sys
 import time
 import uuid
+from enum import Enum
 from logging import getLogger
 from threading import Lock, Timer
-from typing import IO, TYPE_CHECKING, Dict, List, Optional, Tuple, Type, Union
+from typing import (
+    IO,
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    Dict,
+    Generator,
+    Iterator,
+    List,
+    NamedTuple,
+    Optional,
+    Sequence,
+    Tuple,
+    Union,
+)
+
+from snowflake.connector.result_batch import create_batches_from_response
+from snowflake.connector.result_set import ResultSet
 
 from .bind_upload_agent import BindUploadAgent, BindUploadError
 from .compat import BASE_EXCEPTION_CLASS
 from .constants import (
     FIELD_NAME_TO_ID,
     PARAMETER_PYTHON_CONNECTOR_QUERY_RESULT_FORMAT,
+    FileTransferType,
     QueryStatus,
 )
 from .errorcode import (
     ER_CURSOR_IS_CLOSED,
+    ER_FAILED_PROCESSING_PYFORMAT,
     ER_FAILED_TO_REWRITE_MULTI_ROW_INSERT,
     ER_INVALID_VALUE,
     ER_NO_ARROW_RESULT,
@@ -39,7 +59,7 @@ from .errors import (
     ProgrammingError,
 )
 from .file_transfer_agent import SnowflakeFileTransferAgent
-from .json_result import DictJsonResult, JsonResult
+from .options import installed_pandas, pandas
 from .sqlstate import SQLSTATE_FEATURE_NOT_SUPPORTED
 from .telemetry import TelemetryData, TelemetryField
 from .time_util import get_time_millis
@@ -47,25 +67,26 @@ from .time_util import get_time_millis
 if TYPE_CHECKING:  # pragma: no cover
     from .connection import SnowflakeConnection
     from .file_transfer_agent import SnowflakeProgressPercentage
+    from .result_batch import ResultBatch
+
 
 logger = getLogger(__name__)
 
-try:
-    import pyarrow
-except ImportError:
+if installed_pandas:
+    from pyarrow import Table
+else:
     logger.debug("Failed to import pyarrow. Cannot use pandas fetch API")
-    pyarrow = None
+    Table = None
 
 try:
-    from .arrow_result import ArrowResult
+    from .arrow_iterator import PyArrowIterator  # NOQA
 
-    CAN_USE_ARROW_RESULT = True
+    CAN_USE_ARROW_RESULT_FORMAT = True
 except ImportError as e:  # pragma: no cover
     logger.debug(
-        "Failed to import ArrowResult. No Apache Arrow result set format can be used. ImportError: %s",
-        e,
+        f"Failed to import ArrowResult. No Apache Arrow result set format can be used. ImportError: {e}",
     )
-    CAN_USE_ARROW_RESULT = False
+    CAN_USE_ARROW_RESULT_FORMAT = False
 
 STATEMENT_TYPE_ID_DML = 0x3000
 STATEMENT_TYPE_ID_INSERT = STATEMENT_TYPE_ID_DML + 0x100
@@ -94,18 +115,48 @@ ASYNC_RETRY_PATTERN = [1, 1, 2, 3, 4, 8, 10]
 INCIDENT_BLACKLIST = (KeyError, ValueError, TypeError)
 
 
-def exit_handler(*_):  # pragma: no cover
+class ResultMetadata(NamedTuple):
+    name: str
+    type_code: int
+    display_size: int
+    internal_size: int
+    precision: int
+    scale: int
+    is_nullable: bool
+
+    @classmethod
+    def from_column(cls, col: Dict[str, Any]):
+        """Initializes a ResultMetadata object from the column description in the query response."""
+        return cls(
+            col["name"],
+            FIELD_NAME_TO_ID[col["type"].upper()],
+            None,
+            col["length"],
+            col["precision"],
+            col["scale"],
+            col["nullable"],
+        )
+
+
+# TODO: once we drop 3.6 support the return type becomes NoReturn
+def exit_handler(*_) -> None:  # pragma: no cover
     """Handler for signal. When called, it will raise SystemExit with exit code FORCE_EXIT."""
     print("\nForce exit")
     logger.info("Force exit")
     sys.exit(1)
 
 
-class SnowflakeCursor(object):
+class ResultState(Enum):
+    DEFAULT = 1
+    VALID = 2
+    RESET = 3
+
+
+class SnowflakeCursor:
     """Implementation of Cursor object that is returned from Connection.cursor() method.
 
     Attributes:
-        description: tuple of name, type_code, display_size, internal_size, precisio, scale, null_ok
+        description: A list of namedtuples about metadata for all columns.
         rowcount: The number of records updated or selected. If not clear, -1 is returned.
         rownumber: The current 0-based index of the cursor in the result set or None if the index cannot be
             determined.
@@ -141,26 +192,41 @@ class SnowflakeCursor(object):
         flags=re.IGNORECASE | re.MULTILINE | re.DOTALL,
     )
 
+    @staticmethod
+    def get_file_transfer_type(sql: str) -> Optional[FileTransferType]:
+        """Decide whether a SQL is a file transfer and return its type.
+
+        None is returned if the SQL isn't a file transfer so that this function can be
+        used in an if-statement.
+        """
+        if SnowflakeCursor.PUT_SQL_RE.match(sql):
+            return FileTransferType.PUT
+        elif SnowflakeCursor.GET_SQL_RE.match(sql):
+            return FileTransferType.GET
+        return None
+
     def __init__(
         self,
         connection: "SnowflakeConnection",
         use_dict_result: bool = False,
-        json_result_class: Type["JsonResult"] = JsonResult,
-    ):
+    ) -> None:
         """Inits a SnowflakeCursor with a connection.
 
         Args:
             connection: The connection that created this cursor.
-            use_dict_result: Decides whether to use dict result or not. This variable only applied to
-                arrow result. When result in json, json_result_class will be honored.
-            json_result_class: The class that used in json result.
+            use_dict_result: Decides whether to use dict result or not.
         """
-        self._connection = connection
+        self._connection: "SnowflakeConnection" = connection
 
-        self._errorhandler = Error.default_errorhandler
-        self.messages = []
-        self._timebomb = None  # must be here for abort_exit method
-        self._description = None
+        self._errorhandler: Callable[
+            ["SnowflakeConnection", "SnowflakeCursor", Type["Error"], Dict[str, str]],
+            None,
+        ] = Error.default_errorhandler
+        self.messages: List[
+            Tuple[Union[Type["Error"], Type[Exception]], Dict[str, Union[str, bool]]]
+        ] = []
+        self._timebomb: Optional[Timer] = None  # must be here for abort_exit method
+        self._description: Optional[List[ResultMetadata]] = None
         self._column_idx_to_name = None
         self._sfqid = None
         self._sqlstate = None
@@ -177,9 +243,12 @@ class SnowflakeCursor(object):
         self._time_output_format = None
         self._timezone = None
         self._binary_output_format = None
-        self._result = None
+        self._result: Optional[Union[Iterator[Tuple], Iterator[Dict]]] = None
+        self._result_set: Optional["ResultSet"] = None
+        self._result_state: ResultState = ResultState.DEFAULT
         self._use_dict_result = use_dict_result
-        self._json_result_class = json_result_class
+        # TODO: self._query_result_format could be defined as an enum
+        self._query_result_format: Optional[str] = None
 
         self._arraysize = 1  # PEP-0249: defaults to 1
 
@@ -188,12 +257,13 @@ class SnowflakeCursor(object):
         self._first_chunk_time = None
 
         self._log_max_query_length = connection.log_max_query_length
-        self._inner_cursor = None
+        self._inner_cursor: Optional["SnowflakeCursor"] = None
         self._prefetch_hook = None
+        self._rownumber: Optional[int] = None
 
         self.reset()
 
-    def __del__(self):  # pragma: no cover
+    def __del__(self) -> None:  # pragma: no cover
         try:
             self.close()
         except BASE_EXCEPTION_CLASS as e:
@@ -201,7 +271,7 @@ class SnowflakeCursor(object):
                 logger.info(e)
 
     @property
-    def description(self):
+    def description(self) -> List[ResultMetadata]:
         return self._description
 
     @property
@@ -210,9 +280,7 @@ class SnowflakeCursor(object):
 
     @property
     def rownumber(self):
-        return (
-            self._result.total_row_index if self._result.total_row_index >= 0 else None
-        )
+        return self._rownumber if self._rownumber >= 0 else None
 
     @property
     def sfqid(self):
@@ -335,6 +403,7 @@ class SnowflakeCursor(object):
         binding_params: Union[Tuple, Dict[str, Dict[str, str]]] = None,
         binding_stage: Optional[str] = None,
         is_internal: bool = False,
+        describe_only: bool = False,
         _no_results: bool = False,
         _is_put_get=None,
     ):
@@ -353,7 +422,7 @@ class SnowflakeCursor(object):
 
         # check if current installation include arrow extension or not,
         # if not, we set statement level query result format to be JSON
-        if not CAN_USE_ARROW_RESULT:
+        if not CAN_USE_ARROW_RESULT_FORMAT:
             logger.debug("Cannot use arrow result format, fallback to json format")
             if statement_params is None:
                 statement_params = {
@@ -439,6 +508,7 @@ class SnowflakeCursor(object):
                 is_file_transfer=bool(self._is_file_transfer),
                 statement_params=statement_params,
                 is_internal=is_internal,
+                describe_only=describe_only,
                 _no_results=_no_results,
             )
         finally:
@@ -487,7 +557,7 @@ class SnowflakeCursor(object):
     def execute(
         self,
         command: str,
-        params: Optional[Union[List, Tuple]] = None,
+        params: Optional[Union[Sequence[Any], Dict[Any, Any]]] = None,
         _bind_stage: Optional[str] = None,
         timeout: Optional[int] = None,
         _exec_async: bool = False,
@@ -501,13 +571,14 @@ class SnowflakeCursor(object):
         _show_progress_bar: bool = True,
         _statement_params: Optional[Dict[str, str]] = None,
         _is_internal: bool = False,
+        _describe_only: bool = False,
         _no_results: bool = False,
         _use_ijson: bool = False,
         _is_put_get: Optional[bool] = None,
         _raise_put_get_error: bool = True,
         _force_put_overwrite: bool = False,
         file_stream: Optional[IO[bytes]] = None,
-    ):
+    ) -> Optional[Union["SnowflakeCursor", None]]:
         """Executes a command/query.
 
         Args:
@@ -525,8 +596,10 @@ class SnowflakeCursor(object):
             _get_callback_output_stream: The output stream a GET command's callback should report on.
             _show_progress_bar: Whether or not to show progress bar.
             _statement_params: Extra information that should be sent to Snowflake with query.
+            _is_internal: This flag indicates whether the query is issued internally by the connector.
+            _describe_only: If true, the query will not be executed but return the schema/description of this query.
             _no_results: This flag tells the back-end to not return the result, just fire the query and return the
-                query id of the running query.
+                response returned by Snowflake's server.
             _use_ijson: This flag doesn't do anything as ijson support has ended.
             _is_put_get: Force decision of this SQL query being a PUT, or GET command. This is detected otherwise.
             _raise_put_get_error: Whether to raise PUT and GET errors.
@@ -535,7 +608,8 @@ class SnowflakeCursor(object):
             file_stream: File-like object to be uploaded with PUT
 
         Returns:
-            A result class with the results in it. This can either be json, or an arrow result class.
+            The cursor itself, or None if some error happened, or the response returned
+            by Snowflake if the _no_results flag is on.
         """
         if _exec_async:
             _no_results = True
@@ -559,6 +633,7 @@ class SnowflakeCursor(object):
             "timeout": timeout,
             "statement_params": _statement_params,
             "is_internal": _is_internal,
+            "describe_only": _describe_only,
             "_no_results": _no_results,
             "_is_put_get": _is_put_get,
         }
@@ -567,13 +642,14 @@ class SnowflakeCursor(object):
             if self._connection.is_pyformat:
                 # pyformat/format paramstyle
                 # client side binding
-                processed_params = self._connection._process_params(params, self)
+                processed_params = self._connection._process_params_pyformat(
+                    params, self
+                )
                 if logger.getEffectiveLevel() <= logging.DEBUG:
                     logger.debug(
-                        "binding: [%s] with input=[%s], processed=[%s]",
-                        self._format_query_for_log(command),
-                        params,
-                        processed_params,
+                        f"binding: [{self._format_query_for_log(command)}] "
+                        f"with input=[{params}], "
+                        f"processed=[{processed_params}]",
                     )
                 if len(processed_params) > 0:
                     query = command % processed_params
@@ -585,6 +661,17 @@ class SnowflakeCursor(object):
                 if _bind_stage:
                     kwargs["binding_stage"] = _bind_stage
                 else:
+                    if params is not None and not isinstance(params, (list, tuple)):
+                        errorvalue = {
+                            "msg": "Binding parameters must be a list: {}".format(
+                                params
+                            ),
+                            "errno": ER_FAILED_PROCESSING_PYFORMAT,
+                        }
+                        Error.errorhandler_wrapper(
+                            self.connection, self, ProgrammingError, errorvalue
+                        )
+
                     kwargs["binding_params"] = self._connection._process_params_qmarks(
                         params, self
                     )
@@ -637,6 +724,7 @@ class SnowflakeCursor(object):
 
             logger.debug("PUT OR GET: %s", self.is_file_transfer)
             if self.is_file_transfer:
+                # Decide whether to use the old, or new code path
                 sf_file_transfer_agent = SnowflakeFileTransferAgent(
                     self,
                     query,
@@ -704,6 +792,19 @@ class SnowflakeCursor(object):
         kwargs["_exec_async"] = True
         return self.execute(*args, **kwargs)
 
+    def describe(self, *args, **kwargs) -> List[ResultMetadata]:
+        """Obtain the schema of the result without executing the query.
+
+        This function takes the same arguments as execute, please refer to that function
+        for documentation.
+
+        Returns:
+            The schema of the result.
+        """
+        kwargs["_describe_only"] = kwargs["_is_internal"] = True
+        self.execute(*args, **kwargs)
+        return self._description
+
     def _format_query_for_log(self, query):
         return self._connection._format_query_for_log(query)
 
@@ -721,34 +822,24 @@ class SnowflakeCursor(object):
         if self._total_rowcount == -1 and not is_dml and data.get("total") is not None:
             self._total_rowcount = data["total"]
 
-        self._description = []
+        self._description: List[ResultMetadata] = [
+            ResultMetadata.from_column(col) for col in data["rowtype"]
+        ]
 
-        for column in data["rowtype"]:
-            type_value = FIELD_NAME_TO_ID[column["type"].upper()]
-            self._description.append(
-                (
-                    column["name"],
-                    type_value,
-                    None,
-                    column["length"],
-                    column["precision"],
-                    column["scale"],
-                    column["nullable"],
-                )
-            )
+        result_chunks = create_batches_from_response(
+            self, self._query_result_format, data, self._description
+        )
 
-        if self._query_result_format == "arrow":
-            self.check_can_use_arrow_resultset()
-            self._result = ArrowResult(
-                data,
-                self,
-                use_dict_result=self._use_dict_result,
-                number_to_decimal=self._connection.arrow_number_to_decimal,
-            )
-        else:
-            self._result = self._json_result_class(data, self)
+        self._result_set = ResultSet(
+            self,
+            result_chunks,
+            self._connection.client_prefetch_threads,
+        )
+        self._rownumber = -1
+        self._result_state = ResultState.VALID
 
-        if is_dml:
+        # don't update the row count when the result is returned from `describe` method
+        if is_dml and "rowset" in data and len(data["rowset"]) > 0:
             updated_rows = 0
             for idx, desc in enumerate(self._description):
                 if (
@@ -767,9 +858,9 @@ class SnowflakeCursor(object):
                 self._total_rowcount += updated_rows
 
     def check_can_use_arrow_resultset(self):
-        global CAN_USE_ARROW_RESULT
+        global CAN_USE_ARROW_RESULT_FORMAT
 
-        if not CAN_USE_ARROW_RESULT:
+        if not CAN_USE_ARROW_RESULT_FORMAT:
             if self._connection.application == "SnowSQL":
                 msg = "Currently SnowSQL doesn't support the result set in Apache Arrow format."
                 errno = ER_NO_PYARROW_SNOWSQL
@@ -788,9 +879,7 @@ class SnowflakeCursor(object):
             )
 
     def check_can_use_pandas(self):
-        global pyarrow
-
-        if pyarrow is None:
+        if not installed_pandas:
             msg = (
                 "Optional dependency: 'pyarrow' is not installed, please see the following link for install "
                 "instructions: https://docs.snowflake.com/en/user-guide/python-connector-pandas.html#installation"
@@ -843,38 +932,68 @@ class SnowflakeCursor(object):
             )
         return self
 
-    def fetch_pandas_batches(self, **kwargs):
-        """Fetches a single Arrow Table."""
-        self.check_can_use_pandas()
-        if self._query_result_format != "arrow":  # TODO: or pandas isn't imported
-            raise NotSupportedError
-        for df in self._result._fetch_pandas_batches(**kwargs):
-            yield df
-
-    def fetch_pandas_all(self, **kwargs):
-        """Fetch Pandas dataframes in batches, where 'batch' refers to Snowflake Chunk."""
-        self.check_can_use_pandas()
+    def fetch_arrow_batches(self) -> Iterator[Table]:
+        self.check_can_use_arrow_resultset()
         if self._query_result_format != "arrow":
             raise NotSupportedError
-        return self._result._fetch_pandas_all(**kwargs)
+        self._log_telemetry_job_data(
+            TelemetryField.ARROW_FETCH_BATCHES, TelemetryData.TRUE
+        )
+        return self._result_set._fetch_arrow_batches()
+
+    def fetch_arrow_all(self) -> Optional[Table]:
+        self.check_can_use_arrow_resultset()
+        if self._query_result_format != "arrow":
+            raise NotSupportedError
+        self._log_telemetry_job_data(TelemetryField.ARROW_FETCH_ALL, TelemetryData.TRUE)
+        return self._result_set._fetch_arrow_all()
+
+    def fetch_pandas_batches(self, **kwargs) -> Iterator["pandas.DataFrame"]:
+        """Fetches a single Arrow Table."""
+        self.check_can_use_pandas()
+        if self._prefetch_hook is not None:
+            self._prefetch_hook()
+            raise NotSupportedError
+        self._log_telemetry_job_data(
+            TelemetryField.PANDAS_FETCH_BATCHES, TelemetryData.TRUE
+        )
+        return self._result_set._fetch_pandas_batches(**kwargs)
+
+    def fetch_pandas_all(self, **kwargs) -> "pandas.DataFrame":
+        """Fetch Pandas dataframes in batches, where 'batch' refers to Snowflake Chunk."""
+        self.check_can_use_pandas()
+        if self._prefetch_hook is not None:
+            self._prefetch_hook()
+        if self._query_result_format != "arrow":
+            raise NotSupportedError
+        self._log_telemetry_job_data(
+            TelemetryField.PANDAS_FETCH_ALL, TelemetryData.TRUE
+        )
+        return self._result_set._fetch_pandas_all(**kwargs)
 
     def abort_query(self, qid):
         url = "/queries/{qid}/abort-request".format(qid=qid)
         ret = self._connection.rest.request(url=url, method="post")
         return ret.get("success")
 
-    def executemany(self, command, seqparams):
+    def executemany(
+        self,
+        command: str,
+        seqparams: Union[Sequence[Any], Dict[str, Any]],
+    ) -> "SnowflakeCursor":
         """Executes a command/query with the given set of parameters sequentially."""
         logger.debug("executing many SQLs/commands")
         command = command.strip(" \t\n\r") if command else None
 
         if len(seqparams) == 0:
-            errorvalue = {
-                "msg": f"No parameters are specified for the command: {command}",
-                "errno": ER_INVALID_VALUE,
-            }
             Error.errorhandler_wrapper(
-                self.connection, self, InterfaceError, errorvalue
+                self.connection,
+                self,
+                InterfaceError,
+                {
+                    "msg": f"No parameters are specified for the command: {command}",
+                    "errno": ER_INVALID_VALUE,
+                },
             )
             return self
 
@@ -884,19 +1003,23 @@ class SnowflakeCursor(object):
                 command_wo_comments = re.sub(self.COMMENT_SQL_RE, "", command)
                 m = self.INSERT_SQL_VALUES_RE.match(command_wo_comments)
                 if not m:
-                    errorvalue = {
-                        "msg": "Failed to rewrite multi-row insert",
-                        "errno": ER_FAILED_TO_REWRITE_MULTI_ROW_INSERT,
-                    }
                     Error.errorhandler_wrapper(
-                        self.connection, self, InterfaceError, errorvalue
+                        self.connection,
+                        self,
+                        InterfaceError,
+                        {
+                            "msg": "Failed to rewrite multi-row insert",
+                            "errno": ER_FAILED_TO_REWRITE_MULTI_ROW_INSERT,
+                        },
                     )
 
                 fmt = m.group(1)
                 values = []
                 for param in seqparams:
-                    logger.debug("parameter: %s", param)
-                    values.append(fmt % self._connection._process_params(param, self))
+                    logger.debug(f"parameter: {param}")
+                    values.append(
+                        fmt % self._connection._process_params_pyformat(param, self)
+                    )
                 command = command.replace(fmt, ",".join(values), 1)
                 self.execute(command)
                 return self
@@ -906,13 +1029,13 @@ class SnowflakeCursor(object):
                 row_size = len(seqparams[0])
                 for row in seqparams:
                     if len(row) != row_size:
-                        errorvalue = {
+                        error_value = {
                             "msg": f"Bulk data size don't match. expected: {row_size}, "
                             f"got: {len(row)}, command: {command}",
                             "errno": ER_INVALID_VALUE,
                         }
                         Error.errorhandler_wrapper(
-                            self.connection, self, InterfaceError, errorvalue
+                            self.connection, self, InterfaceError, error_value
                         )
                         return self
                 bind_size = len(seqparams) * row_size
@@ -932,7 +1055,8 @@ class SnowflakeCursor(object):
                         bind_stage = bind_uploader.stage_path
                     except BindUploadError:
                         logger.debug(
-                            "Failed to upload binds to stage, sending binds to Snowflake instead."
+                            "Failed to upload binds to stage, sending binds to "
+                            "Snowflake instead."
                         )
                     except Exception as exc:
                         if not isinstance(exc, INCIDENT_BLACKLIST):
@@ -949,12 +1073,35 @@ class SnowflakeCursor(object):
             self.execute(command, param, _do_reset=False)
         return self
 
-    def fetchone(self):
+    def _result_iterator(
+        self,
+    ) -> Union[Generator[Dict, None, None], Generator[Tuple, None, None]]:
+        """Yields the elements from _result and raises an exception when appropriate."""
+        try:
+            for _next in self._result:
+                if isinstance(_next, Exception):
+                    Error.errorhandler_wrapper_from_ready_exception(
+                        self._connection,
+                        self,
+                        _next,
+                    )
+                self._rownumber += 1
+                yield _next
+        except TypeError as err:
+            if self._result_state == ResultState.DEFAULT:
+                raise err
+            else:
+                yield None
+
+    def fetchone(self) -> Optional[Union[Dict, Tuple]]:
         """Fetches one row."""
         if self._prefetch_hook is not None:
             self._prefetch_hook()
+        if self._result is None and self._result_set is not None:
+            self._result = iter(self._result_set)
+            self._result_state = ResultState.VALID
         try:
-            return next(self._result)
+            return next(self._result_iterator())
         except StopIteration:
             return None
 
@@ -984,7 +1131,7 @@ class SnowflakeCursor(object):
 
         return ret
 
-    def fetchall(self):
+    def fetchall(self) -> Union[List[Tuple], List[Dict]]:
         """Fetches all of the results."""
         ret = []
         while True:
@@ -1023,17 +1170,25 @@ class SnowflakeCursor(object):
     def reset(self):
         """Resets the result set."""
         self._total_rowcount = -1  # reset the rowcount
+        if self._result_state != ResultState.DEFAULT:
+            self._result_state = ResultState.RESET
         if self._result is not None:
-            self._result._reset()
+            self._result = None
         if self._inner_cursor is not None:
             self._inner_cursor.reset()
             self._result = None
             self._inner_cursor = None
         self._prefetch_hook = None
+        if not self.connection._reuse_results:
+            self._result_set = None
 
-    def __iter__(self):
+    def __iter__(self) -> Union[Iterator[Dict], Iterator[Tuple]]:
         """Iteration over the result set."""
-        return iter(self._result)
+        # set _result if _result_set is not None
+        if self._result is None and self._result_set is not None:
+            self._result = iter(self._result_set)
+            self._result_state = ResultState.VALID
+        return self._result_iterator()
 
     def __cancel_query(self, query):
         if self._sequence_counter >= 0 and not self.is_closed():
@@ -1043,7 +1198,11 @@ class SnowflakeCursor(object):
 
     def _log_telemetry_job_data(self, telemetry_field, value):
         """Builds an instance of TelemetryData with the given field and logs it."""
-        obj = {"type": telemetry_field, "query_id": self._sfqid, "value": int(value)}
+        obj = {
+            "type": telemetry_field,
+            "query_id": self._sfqid,
+            "value": int(value),
+        }
         ts = get_time_millis()
         try:
             self._connection._log_telemetry(TelemetryData(obj, ts))
@@ -1095,6 +1254,12 @@ class SnowflakeCursor(object):
                 "select * from table(result_scan('{}'))".format(sfqid)
             )
             self._result = self._inner_cursor._result
+            self._query_result_format = self._inner_cursor._query_result_format
+            self._total_rowcount = self._inner_cursor._total_rowcount
+            self._description = self._inner_cursor._description
+            self._result_set = self._inner_cursor._result_set
+            self._result_state = ResultState.VALID
+            self._rownumber = 0
             # Unset this function, so that we don't block anymore
             self._prefetch_hook = None
 
@@ -1106,11 +1271,28 @@ class SnowflakeCursor(object):
         self._sfqid = sfqid
         self._prefetch_hook = wait_until_ready
 
+    def get_result_batches(self) -> Optional[List["ResultBatch"]]:
+        """Get the previously executed query's ``ResultBatch`` s if available.
+
+        If they are unavailable, in case nothing has been executed yet None will
+        be returned.
+
+        For a detailed description of ``ResultBatch`` s please see the docstring of:
+        ``snowflake.connector.result_batches.ResultBatch``
+        """
+        if self._result_set is None:
+            return None
+        self._log_telemetry_job_data(
+            TelemetryField.GET_PARTITIONS_USED, TelemetryData.TRUE
+        )
+        return self._result_set.batches
+
 
 class DictCursor(SnowflakeCursor):
     """Cursor returning results in a dictionary."""
 
     def __init__(self, connection):
-        SnowflakeCursor.__init__(
-            self, connection, use_dict_result=True, json_result_class=DictJsonResult
+        super().__init__(
+            connection,
+            use_dict_result=True,
         )
