@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2012-2021 Snowflake Computing Inc. All right reserved.
+# Copyright (c) 2012-2021 Snowflake Computing Inc. All rights reserved.
 #
 
 import os
@@ -204,7 +204,7 @@ def write_pandas(
     return (
         all(e[1] == "LOADED" for e in copy_results),
         len(copy_results),
-        sum(e[3] for e in copy_results),
+        sum(int(e[3]) for e in copy_results),
         copy_results,
     )
 
